/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.base;

import java.io.*;

/**
 * Class <code>FObjectInputStream</code> overloads the class <code>ObjectInputStream</code>
 * to solve the bug reported under BugID
 * <a href="http://developer.java.sun.com/developer/bugParade/bugs/4171142.html">4171142</a> .
 * <br>
 * If an object contains primitiv data types and you try to serialize/deserialize
 * it than you will get an ClassNotFoundException for the primitiv data type.
 * Reason for this exception is that the ObjectInputStream try to resolve the
 * data type from java.lang.&lt;primitiv&gt; which does not work.
 * <br>
 * The solution for the problem is to override the method resolveClass and to
 * bypass the resolution for primitiv data types.
 *
 * @author <a href="mailto:adietzsch@de.ibm.com?subject=FObjectInputStream">Alexander Dietzsch</a>
 */
class WSIFObjectInputStream extends ObjectInputStream {
  /**
   * Create an ObjectInputStream that reads from the specified InputStream.
   * The stream header containing the magic number and version number
   * are read from the stream and verified. This method will block
   * until the corresponding ObjectOutputStream has written and flushed the
   * header.
   *
   * @param in  the underlying <code>InputStream</code> from which to read
   * @exception StreamCorruptedException The version or magic number are
   * incorrect.
   * @exception IOException An exception occurred in the underlying stream.
   */
  WSIFObjectInputStream (InputStream in)
    throws IOException, StreamCorruptedException {

    super(in);
  }
  /**
   * Load the local class equivalent of the specified stream class description.
   *
   * Subclasses may implement this method to allow classes to be
   * fetched from an alternate source.
   *
   * The corresponding method in ObjectOutputStream is
   * annotateClass.  This method will be invoked only once for each
   * unique class in the stream.  This method can be implemented by
   * subclasses to use an alternate loading mechanism but must
   * return a Class object.  Once returned, the serialVersionUID of the
   * class is compared to the serialVersionUID of the serialized class.
   * If there is a mismatch, the deserialization fails and an exception
   * is raised. <p>
   *
   * By default the class name is resolved relative to the class
   * that called readObject. <p>
   *
   * @param v  an instance of class ObjectStreamClass
   * @return a Class object corresponding to <code>v</code>
   * @exception IOException Any of the usual Input/Output exceptions.
   * @exception ClassNotFoundException If class of
   * a serialized object cannot be found.
   */
  protected Class resolveClass(ObjectStreamClass v)
    throws IOException, ClassNotFoundException {

    try {
      // return super.resolveClass(v);
      return Class.forName(v.getName(), true, Thread.currentThread().getContextClassLoader());
    } catch (ClassNotFoundException xcpt) {
	 String className = v.getName() ;
      if (className.equals("boolean")) { return boolean.class; }
      if (className.equals("char")) { return char.class; }
      if (className.equals("byte")) { return byte.class; }
      if (className.equals("short")) { return short.class; }
      if (className.equals("int")) { return int.class; }
      if (className.equals("long")) { return long.class; }
      if (className.equals("float")) { return float.class; }
      if (className.equals("double")) { return double.class; }

      throw xcpt;
    }
  }
}
