/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.java;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.xml.namespace.QName;

import org.apache.wsif.WSIFException;
import org.apache.wsif.WSIFOperation;
import org.apache.wsif.base.WSIFDefaultPort;
import org.apache.wsif.logging.Trc;
import org.apache.wsif.providers.WSIFDynamicTypeMap;
import org.apache.wsif.util.WSIFUtils;
import org.apache.wsif.wsdl.extensions.format.TypeMap;
import org.apache.wsif.wsdl.extensions.format.TypeMapping;
import org.apache.wsif.wsdl.extensions.java.JavaAddress;

/**
 * Java WSIF Port.
 * @author <a href="mailto:gpfau@de.ibm.com">Gerhard Pfau</a>
 * Partially based on WSIFPort_ApacheSOAP from Alekander Slominski, 
 * Paul Fremantle, Sanjiva Weerawarana and Matthew J. Duftler
 * @author Owen Burroughs <owenb@apache.org>
 * @author Jeremy Hughes <hughesj@apache.org>
 */
public class WSIFPort_Java extends WSIFDefaultPort implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
    private javax.wsdl.Definition fieldDefinition = null;
    private javax.wsdl.Port fieldPortModel = null;
    private java.lang.Object fieldObjectReference = null; // 'physical connection'
    private Map fieldTypeMaps = new HashMap();

    protected Map operationInstances = new HashMap();

    public WSIFPort_Java(Definition def, Port port, WSIFDynamicTypeMap typeMap)
        throws WSIFException {
        Trc.entry(this, def, port, typeMap);
        fieldDefinition = def;
        fieldPortModel = port;
        buildTypeMap();
        if (Trc.ON)
            Trc.exit(deep());
    }

    public Definition getDefinition() {
        Trc.entry(this);
        Trc.exit(fieldDefinition);
        return fieldDefinition;
    }

    public WSIFOperation createOperation(String operationName)
        throws WSIFException {
        Trc.entry(this);
        WSIFOperation wo = createOperation(operationName, null, null);
        Trc.exit(wo);
        return wo;
    }

    public WSIFOperation createOperation(
        String operationName,
        String inputName,
        String outputName)
        throws WSIFException {
        Trc.entry(this, operationName, inputName, outputName);

        WSIFOperation_Java op =
            getDynamicWSIFOperation(operationName, inputName, outputName);
        if (op == null) {
            throw new WSIFException(
                "Could not create operation: "
                    + operationName
                    + ":"
                    + inputName
                    + ":"
                    + outputName);
        }
        WSIFOperation wo = op.copy();
        Trc.exit(wo);
        return wo;
    }

    protected WSIFOperation_Java getDynamicWSIFOperation(
        String name,
        String inputName,
        String outputName)
        throws WSIFException {
        Trc.entry(this, name, inputName, outputName);

        WSIFOperation_Java operation =
            (WSIFOperation_Java) operationInstances.get(
                getKey(name, inputName, outputName));

        if (operation == null) {
            BindingOperation bindingOperationModel =
            WSIFUtils.getBindingOperation( 
               fieldPortModel.getBinding(), name, inputName, outputName );

            if (bindingOperationModel != null) {
                operation =
                    new WSIFOperation_Java(
                        fieldPortModel,
                        bindingOperationModel,
                        this,
                        fieldTypeMaps);
                setDynamicWSIFOperation(name, inputName, outputName, operation);
            }
        }
        Trc.exit(operation);
        return operation;
    }

    public java.lang.Object getObjectReference() throws WSIFException {
        Trc.entry(this);
        if (fieldObjectReference == null) {
            JavaAddress address = null;

            try {
                ExtensibilityElement portExtension =
                    (ExtensibilityElement) fieldPortModel.getExtensibilityElements().get(0);

                if (portExtension == null) {
                    throw new WSIFException("missing port extension");
                }

                address = (JavaAddress) portExtension;

                fieldObjectReference =
                    Class
                        .forName(
                            address.getClassName(),
                            true,
                            Thread.currentThread().getContextClassLoader())
                        .newInstance();
            } catch (Exception ex) {
            	Trc.exception(ex);
                throw new WSIFException(
                    "Could not create object of class '" + address.getClassName() + "'",
                    ex);
            }
        }
        Trc.exit(fieldObjectReference);
        return fieldObjectReference;
    }

    public Port getPortModel() {
        Trc.entry(this);
        Trc.exit(fieldPortModel);
        return fieldPortModel;
    }

    public void setDefinition(Definition value) {
        Trc.entry(this, value);
        fieldDefinition = value;
        Trc.exit();
    }

    // WSIF: keep list of operations available in this port
    public void setDynamicWSIFOperation(
        String name,
        String inputName,
        String outputName,
        WSIFOperation_Java value) {
        Trc.entry(this, name, inputName, outputName, value);
        operationInstances.put(getKey(name, inputName, outputName), value);
        Trc.exit();
    }

    public void setObjectReference(java.lang.Object newObjectReference) {
        Trc.entry(this, newObjectReference);
        fieldObjectReference = newObjectReference;
        Trc.exit();
    }

    public void setPortModel(Port value) {
        Trc.entry(this, value);
        fieldPortModel = value;
        Trc.exit();
    }

    private void buildTypeMap() throws WSIFException {
    	Trc.entry(this);
        TypeMapping typeMapping = null;

        // Get the TypeMappings from the binding
        Iterator bindingIterator =
            this.fieldPortModel.getBinding().getExtensibilityElements().iterator();
            
        // Choose the first typeMap that has encoding=Java and style=Java. 
        // Ignore any other typeMap's that have other encodings and styles.
        while (bindingIterator.hasNext()) {
            Object next = bindingIterator.next();
            if (next instanceof TypeMapping) {
                typeMapping = (TypeMapping) next;
                if ("Java".equals(typeMapping.getEncoding()) &&
                    "Java".equals(typeMapping.getStyle()))
                    break;
                typeMapping = null;
            }
        }

        if (typeMapping == null) {
            QName bindingName = fieldPortModel.getBinding().getQName();
            throw new WSIFException(
                "Binding "
                    + (bindingName == null ? "<null>" : bindingName.toString())
                    + " does not contain a typeMap with encoding=Java and style=Java");
        }

        // Build the hashmap 
        bindingIterator = typeMapping.getMaps().iterator();
        while (bindingIterator.hasNext()) {
            TypeMap typeMap = (TypeMap) bindingIterator.next();
            ///////////////////////////////////
            QName typeName = typeMap.getTypeName();
            if (typeName == null) typeName = typeMap.getElementName();
            String type = typeMap.getFormatType();
            if (typeName != null && type != null) {
                if (fieldTypeMaps.containsKey(typeName)) {
                	Vector v = null;
                	Object obj = fieldTypeMaps.get(typeName);                	
                	if (obj instanceof Vector) {
                		v = (Vector) obj;
                	} else {
                		v = new Vector();
	                    v.addElement(obj);
                	}                	                    
                    v.addElement(type);
                    this.fieldTypeMaps.put(typeName, v);
                } else {
                    this.fieldTypeMaps.put(typeName, type);
                }
            } else {
                throw new WSIFException("Error in binding TypeMap. Key or Value is null");
            }
        }
        Trc.exit();
    }

    public String deep() {
        String buff = "";
        try {
            buff = new String(super.toString() + ":\n");

            buff += "definition:" + fieldDefinition == null
                ? "null"
                : fieldDefinition.getQName() == null
                ? "unknown"
                : fieldDefinition.getQName().toString();
            buff += " portModel:" + fieldPortModel == null
                ? "null"
                : fieldPortModel.getName() == null
                ? "unknown"
                : fieldPortModel.getName().toString();
            buff += " objectReference:" + fieldObjectReference;

            buff += " operationInstances:";
            if (operationInstances == null)
                buff += "null";
            else {
                buff += " size:" + operationInstances.size();
                Iterator it = operationInstances.keySet().iterator();
                int i = 0;
                while (it.hasNext()) {
                    String key = (String) it.next();
                    WSIFOperation_Java woj = (WSIFOperation_Java) operationInstances.get(key);
                    buff += "\noperationInstances[" + i + "]:" + key + " " + woj + " ";
                    i++;
                }
            }
        } catch (Exception e) {
            Trc.exceptionInTrace(e);
        }

        return buff;
    }
}