/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */


package org.apache.wsif.providers.jca;

import javax.wsdl.*;
import javax.resource.cci.*;
import org.apache.wsif.*;
import org.apache.wsif.providers.WSIFDynamicTypeMap;

/**
 * This interface contains methods implemented by each Resource Adapter and used by the Connector 
 * Architecture provider to delegate Connector specific operations, for example creation of the
 * WSIFOperation to the Resource Adapter.
 * 
 * @author Michael Beisiegel
 * @author Piotr Przybylski <piotrp@ca.ibm.com>
 * @author John Green
 */
public interface WSIFProviderJCAExtensions {
	

	/**
	 * The provider for a resource adapter creates a WSIFOperation based on the specified WSDL
	 * operation.  The binding operation extensibility element allows the resource adapter to
	 * populate its InteractionSpec to be used in the operation.
	 * 
	 * @param definition
	 * @param aService
	 * @param aPort
	 * @param operationName
	 * @param inputName
	 * @param outputName
	 * @param typeMap
	 * @param jcaPort
	 * @param connection
	 * @return WSIFOperation
	 * @throws WSIFException
	 */
	public WSIFOperation createOperation(Definition definition, Service aService, Port aPort, String operationName, String inputName, String outputName, WSIFDynamicTypeMap typeMap, WSIFPort_JCA jcaPort, Connection connection) throws WSIFException;
	/**
	 * This method creates input message. It only needs to be implemented by Resource Adapter which
	 * uses custom format of the input and output records (i.e. does not use javax.resource.cci.Streamable
	 * interface).
	 * 
	 * @param definition
	 * @param binding
	 * @param operationName
	 * @param inputName
	 * @param outputName
	 * @return WSIFMessage
	 */
	public WSIFMessage createInputMessage(Definition definition, Binding binding, String operationName, String inputName, String outputName);
	/**
	 * This method creates output message. It only needs to be implemented by Resource Adapter which
	 * uses custom format of the input and output records (i.e. does not use javax.resource.cci.Streamable
	 * interface).
	 * 
	 * @param definition
	 * @param binding
	 * @param operationName
	 * @param inputName
	 * @param outputName
	 * @return WSIFMessage
	 */
	public WSIFMessage createOutputMessage(Definition definition, Binding binding, String operationName, String inputName, String outputName);
	/**
	 * This method creates a FaultMessage.
	 * 
	 * @param definition
	 * @param binding
	 * @param operationName
	 * @param inputName
	 * @param outputName
	 * @return WSIFMessage
	 */
	public WSIFMessage createFaultMessage(Definition definition, Binding binding, String operationName, String inputName, String outputName);
	/**
	 * Updates the interactionSpec from input message values. The method is called from
	 * within the operation execute method, before invocation of Interaction.execute() method
	 * of the resource adapter.
	 * 
	 * @param input
	 * @param aBinding
	 * @param aOperationName
	 * @param aInputName
	 * @param aOutputName
	 * @param aInteractionSpec
	 * @throws WSIFException
	 */
	public void updateInteractionSpec(WSIFMessage input, Binding aBinding, String aOperationName, String aInputName, String aOutputName, InteractionSpec aInteractionSpec) throws WSIFException;
	/**
	 * Updates the output message using output InteractionSpec values.  This method is called
	 * from within the WSIFOperation execute method, after processing the Interaction.execute() method
	 * of the resource adapter..	 
	 * 
	 * @param output Output message to populate
	 * @param aBinding Bonding
	 * @param aOperationName Operation name
	 * @param aInputName Inpput name
	 * @param aOutputName Output name
	 * @param aInteractionSpec InteractionSpec after the execute() method invocation 
	 * @throws WSIFException
	 */
	public void updateOutputMessage(WSIFMessage output, Binding aBinding, String aOperationName, String aInputName, String aOutputName, InteractionSpec aInteractionSpec) throws WSIFException;
	/**
	 * Creates a javax.resource.cci.Connection.  This should be used when a resource adapter supports
	 * passing ConnectionSpec values as part of the input message.  WSIFOperation_JCA will only call
	 * this method during the execute method if the WSIFPort_JCA does not contain a connection.
	 * 
	 * @param input
	 * @param definition
	 * @param service
	 * @param port
	 * @param typeMap
	 * @param aBinding
	 * @param aOperationName
	 * @param aInputName
	 * @param aOutputName
	 * @return Connection
	 * @throws WSIFException
	 */
	public Connection createConnection(WSIFMessage input, Definition definition, Service service, Port port, org.apache.wsif.providers.WSIFDynamicTypeMap typeMap, Binding aBinding, String aOperationName, String aInputName, String aOutputName) throws WSIFException;
}
